<?php
    /* vim: set expandtab tabstop=4 shiftwidth=4: */
    // +----------------------------------------------------------------------+
    // | PHP version 5                                                        |
    // +----------------------------------------------------------------------+
    // | Copyright (c) 1997-2004 The PHP Group                                |
    // +----------------------------------------------------------------------+
    // | This source file is subject to version 3.0 of the PHP license,       |
    // | that is bundled with this package in the file LICENSE, and is        |
    // | available through the world-wide-web at the following url:           |
    // | http://www.php.net/license/3_0.txt.                                  |
    // | If you did not receive a copy of the PHP license and are unable to   |
    // | obtain it through the world-wide-web, please send a note to          |
    // | license@php.net so we can mail you a copy immediately.               |
    // +----------------------------------------------------------------------+
    // | Authors: Claudio Bustos <cdx@users.sourceforge.net>                  |
    // |          Jens Bierkandt <schtorch@users.sourceforge.net>             |
    // +----------------------------------------------------------------------+
    //
    // $Id:
    /**
    * Definition of  PHP_Beautifier_Batch
    * @package PHP_Beautifier
    * @subpackage Batch
    * @author Claudio Bustos <cdx@users.sourceforge.net>
    */
    /**
    * Require PHP_Beautifier_Decorator
    */
    require_once ('Decorator.php');
    /**
    * @package PHP_Beautifier
    * @subpackage Batch
    */
    // ArrayNested->off();
    abstract class PHP_Beautifier_Batch_Engine {
        protected $oBatch;
        public function __construct(PHP_Beautifier_Batch $oBatch) 
        {
            $this->oBatch = $oBatch;
        }
        protected function beautifierSetInputFile($sFile) 
        {
            return $this->oBatch->callBeautifier($this, 'setInputFile', array($sFile));
        }
        protected function beautifierProcess() 
        {
            return $this->oBatch->callBeautifier($this, 'process');
        }
        protected function beautifierGet() 
        {
            return $this->oBatch->callBeautifier($this, 'get');
        }
        protected function beautifierSave($sFile) 
        {
            return $this->oBatch->callBeautifier($this, 'save', array($sFile));
        }
        public function get() 
        {
        }
        public function save() 
        {
        }
    }
    // ArrayNested->on();
    /**
    * Adds functionality to handle multiple files.
    * - STDIN  : As normal
    * - STDOUT : Send all the scripts, prepended with the name of the original route
    * - One in, one out: as normal
    * - Multiple In, one out: determine the type of out.
    *   - Without '/' at the end, same as STDOUT.
    *   - With '/' at the end, copy the base structure and copy all the scripts
    *
    * If the file out end in .tgz, the output will be a tar archive. The same action
    * will be obtained with {@link setCompress()} to true
    * Use:
    * <code>
    * require "PHP/Beautifier.php";
    * require "PHP/Beautifier/Batch.php";
    * $oBeaut= new PHP_Beautifier();
    * $oBatch= new PHP_Beautifier_Batch($oBeaut); // Decorator
    * $oBatch->setInputFile(__FILE__);
    * $oBatch->process();
    * $oBatch->show();
    * </code>
    * @package PHP_Beautifier
    * @subpackage Batch
    * @author Claudio Bustos <cdx@users.sourceforge.net>
    */
    class PHP_Beautifier_Batch extends PHP_Beautifier_Decorator {
        private $bStdin = false;
        private $bStdout = false;
        private $sCompress = '';
        private $mPreInputFiles = array();
        private $sPreOutputFile = './';
        private $mInputFiles;
        private $mOutputFiles;
        private $sOutputMode;
        const FILES = 'Files';
        const DIRECTORY = 'Directory';
        public $bRecursive = false;
        // public methods, overloaded from PHP_Beautifier
        /**
        * Set recursive search for files in dirs on
        * @param bool
        */
        public function setRecursive($bRecursive = true) 
        {
            $this->bRecursive = $bRecursive;
        }
        /**
        * Set compression on/off
        * @param mixed bool(false, true for gzip) or string ('gz' or 'gz2')
        */
        public function setCompress($mCompress = true) 
        {
            if ($mCompress === true) {
                $mCompress = 'gz';
            }
            if (!is_string($mCompress)) {
                throw(new Exception('You have to define a mode for compress'));
            }
            $this->sCompress = $mCompress;
        }
        /**
        * Set the input(s) files
        * Could be STDIN or a name, with special chars (?,*)
        * @param mixed STDIN or string(path)
        * @return bool
        */
        public function setInputFile($mFiles) 
        {
            if ($this->mPreInputFiles == STDIN and $mFiles != STDIN) {
                throw(new Exception("Hey, you already defined STDIN,dude"));
            } elseif ($mFiles == STDIN) {
                $this->mPreInputFiles = STDIN;
            } else {
                if (is_string($mFiles)) {
                    $mFiles = array(
                        $mFiles
                    );
                }
                $this->mPreInputFiles = array_merge($this->mPreInputFiles, $mFiles);
            }
            return true;
        }
        /**
        * Set the output file
        * Could be STDOUT or a path to a file or dir (with '/' at the end)
        * @param mixed STDOUT or string (path)
        * @return true
        */
        public function setOutputFile($sFile) 
        {
            if (!is_string($sFile) and $sFile != STDOUT) {
                throw(new Exception("Accept only string or STDOUT"));
            }
            $this->sPreOutputFile = $sFile;
            return true;
        }
        private function setInputFilePost() 
        {
            if ($this->mPreInputFiles == STDIN) {
                $this->mInputFiles = array(
                    STDIN
                );
            } else {
                $this->mInputFiles = array();
                foreach($this->mPreInputFiles as $sPath) {
                    $this->mInputFiles = array_merge($this->mInputFiles, $this->getFilesByGlob($sPath, $this->bRecursive));
                }
            }
            return true;
        }
        private function setOutputFilePost() 
        {
            if ($this->sPreOutputFile == STDOUT) {
                $this->sOutputMode = PHP_Beautifier_Batch::FILES;
            } else {
                $sPath = str_replace(DIRECTORY_SEPARATOR, '/', $this->sPreOutputFile);
                if (!$sPath) {
                    $sPath = "./";
                }
                // determine file or dir
                if (substr($sPath, -1) != '/' and !is_dir($sPath)) {
                    $this->sOutputMode = PHP_Beautifier_Batch::FILES;
                    // Define compression mode
                    if (preg_match("/\.(gz|bz2)$/", $sPath, $aMatch)) {
                        $this->sCompress = $aMatch[1];
                    } 
                } else {
                    $this->sOutputMode = PHP_Beautifier_Batch::DIRECTORY;
                }
            }
            return true;
        }
        /**
        * Create the real references to files
        * @return bool
        * @throws Exception
        */
        public function process() 
        {
            if (!$this->mPreInputFiles) {
                throw(new Exception('Input file not defined'));
            } else {
                $this->setInputFilePost();
                $this->setOutputFilePost();
            }
            if (!$this->mInputFiles) {
                throw(new Exception(implode(',', $this->mPreInputFiles) ." pattern doesn't match any files"));
            } elseif (!$this->mOutputFiles) {
                //throw(new Exception("Can't define output Files"));
            } else {
                return true;
            }
        }
        private function getBatchEngine() 
        {
            $sCompress = ucfirst($this->sCompress);
            $sClass = $this->sOutputMode.$sCompress;
            $sClassEngine = 'PHP_Beautifier_Batch_'.$this->sOutputMode.$sCompress;
            $sClassFile = PHP_Beautifier_Common::normalizeDir(dirname(__FILE__)) .'Batch/'.$sClass.'.php';
            if (!file_exists($sClassFile)) {
                throw(new Exception("$sClassFile doesn't exists"));
            } else {
                include_once ($sClassFile);
                if (!class_exists($sClassEngine)) {
                    throw(new Exception("$sClassFile exists, but $sClassEngine isn't defined"));
                } else {
                    return new $sClassEngine($this);
                }
            }
        }
        /**
        * Save the beautified sources to file(s)
        * @return bool
        * @throws Exception
        */
        public function save() 
        {
            $oBatchEngine = $this->getBatchEngine();
            return $oBatchEngine->save();
        }
        /**
        * Return a string with the content of the file(s)
        * @return string
        */
        public function get() 
        {
            $oBatchEngine = $this->getBatchEngine();
            return $oBatchEngine->get();
        }
        public function show() 
        {
            echo $this->get();
        }
        private function saveMultipleInMultipleOut() 
        {
            for ($i = 0;$i<count($this->mInputFiles);$i++) {
                $this->oBeaut->setInputFile($this->mInputFiles[$i]);
                $this->oBeaut->setOutputFile($this->mOutputFiles[$i]);
                PHP_Beautifier_Common::createDir($this->mOutputFiles[$i]);
                $this->oBeaut->process();
                $this->oBeaut->save();
            }
        }
        /**
        * Allows subclass of {@link PHP_Beautifier_Batch_Engine} call methods of {@link $oBeaut}
        * @param    PHP_Beautifier_Batch_Engine
        * @param    string method to call
        * @param    array  array of args
        * @return   mixed
        */
        public function callBeautifier(PHP_Beautifier_Batch_Engine $oEngine, $sMethod, $aArgs = array()) 
        {
            return call_user_func_array(array(
                $this->oBeaut,
                $sMethod
            ) , $aArgs);
        }
        private function saveMultipleInMultipleOutCompress() 
        {
            include_once 'Archive/Tar.php';
            for ($i = 0;$i<count($this->mInputFiles);$i++) {
                $this->oBeaut->setInputFile($this->mInputFiles[$i]);
                PHP_Beautifier_Common::createDir($this->mOutputFiles[$i]);
                $this->oBeaut->process();
                $sExt = ($this->mCompress === TRUE) ? 'tgz' : $this->mCompress;
                $oArchive = new Archive_Tar($this->mOutputFiles[$i].'.'.$sExt, $this->mCompress);
                $oArchive->addString($this->mInputFiles[$i], $this->oBeaut->get());
            }
        }
        private function saveMultipleInOneOutCompress() 
        {
            include_once 'Archive/Tar.php';
            $oArchive = new Archive_Tar($this->sPreOutputFile, $this->mCompress);
            for ($i = 0;$i<count($this->mInputFiles);$i++) {
                $this->oBeaut->setInputFile($this->mInputFiles[$i]);
                $this->oBeaut->process();
                $oArchive->addString($this->mInputFiles[$i], $this->oBeaut->get());
            }
        }
        private function saveMultipleInOneOut() 
        {
            $sOutFile = $this->mOutputFiles[0];
            if ($sOutFile == STDOUT) {
                $fp = STDOUT;
            } else {
                PHP_Beautifier_Common::createDir(basename($sOutFile));
                $fp = fopen($sOutFile, 'w');
            }
            for ($i = 0;$i<count($this->mInputFiles);$i++) {
                $sText = $this->getWithHeader($this->mInputFiles[$i]);
                fputs($fp, $sText, strlen($sText));
            }
            if ($sOutFile != STDOUT) {
                fclose($fp);
            }
        }
        private function getWithHeader($sFile) 
        {
            $sError = false;
            try {
                $this->oBeaut->setInputFile($sFile);
                $this->oBeaut->process();
            }
            catch(Exception $oExp) {
                $sError = $oExp->getMessage();
            }
            $sHeader = '- BEGIN OF '.$sFile.' -'.$this->oBeaut->sNewLine;
            $sLine = str_repeat('-', strlen($sHeader) -1) .$this->oBeaut->sNewLine;
            $sEnd = '- END OF '.$sFile.str_repeat(' ', strlen($sHeader) -strlen($sFile) -12) .' -'.$this->oBeaut->sNewLine;
            $sText = $sLine.$sHeader.$sLine.$this->oBeaut->sNewLine;
            $sText.= ($sError) ? $sError : $this->oBeaut->get();
            $sText.= $this->oBeaut->sNewLine.$sLine.$sEnd.$sLine.$this->oBeaut->sNewLine;
            return $sText;
        }
        private function getFilesByGlob($sPath, $bRecursive = false) 
        {
            // first, replace * and ?
            // ArrayNested->off();
            $aFind = array(DIRECTORY_SEPARATOR, '.', '?', '*');
            $aReplace = array('/', '\.', '.', '.*?');
            // ArrayNested->on();
            $sDir = dirname($sPath);
            $sFilePattern = str_replace($aFind, $aReplace, basename($sPath));
            // ok, we got the base dir. Now, got the files!
            return PHP_Beautifier_Common::getFilesByPattern($sDir, $sFilePattern, $bRecursive);
        }
        public function getInputFiles() 
        {
            return $this->mInputFiles;
        }
        public function getOutputPath() 
        {
            return $this->sPreOutputFile;
        }
    }
?>